#!/bin/bash
# Copyright PWSCF, Eyvaz Isaev, 2004-2005
# eyvaz_isaev@yahoo.com, e.isaev@misis.ru
# Theoretical Physics Department
# Moscow State Institute of Steel and Alloys
#
# This script is designed for the Fermi Surface construction using XCrySDen thanks
# Tone Kokalj. See www.xcrysden.org for more details. You can download it from the website.
# Two another programs are required to run the script in order to get an input
# file in bxsf-format which could be used directly, without any changes, with 
# XCrySDen. These  auxialiary files are placed in /pwtools directory and could be compiled 
# using "make tools".
# The first additional file, kvecs_FS.f,  is used for k-points generation.
# The second file, bands_FS.f, is required to reorder bands. 
# A user has to edit so-called "user part" in order to define some required parameters.
#
# They are quite simple:
# 
# Sysname   - a nickname for your system
# Calc_Type - The Fermi Surface calculations (FS) or band-structure calculations (Band) which 
#             will be included later. Presently band-structure calculations could be carried out 
#             by means of plotband.x from PP (postprocessing) directory or a little package 
#             distributed by E.Isaev (at least I sent it to the forum). 
# nabc      - a number for dividing of each edge of a parallelepiped. Be careful, the total number 
#             of generated k-points is (na+1)*(nb+1)*(nc+1), i.e. including \Gamma-point.
# n_start   - starting band's number for the Fermi Surface calculations. It is obvious, we have
#             to deal with the bands crossing the Fermi level.
# n_last    - last band's number for FS calculations
#
# That's all!!!
#
# Nota Bene : You can take more bands and then choose from a XCrySDen menu only these bands  
#             which cross the Fermi level   
# Nota Bene : If you have mistaken choosing bands to be considered for the FS construction, 
#             you do not need to restart all calculations. Just edit "bands_FS" file and 
#             restart "bands_FS.x" manually. It will read Bands.out and result Bands.bxsf
#             which you can rename as you like.
#  
# Sure you have to edit input files for self-consistent and non-self-consistent
# calculations for your own purpose.
# Please keep untouched input files "input_kvecs" and "input_FS". There is no need to edit them. 
#
# Spin-polarized calculations are not allowed, but will be done soon.

# begin of user part
#
Sysname='Al'
Calc_Type='FS'
nabc=' 24 24 24 '
n_start=2
n_last=5
#
# end of user part
#

. ./environment_variables

echo $Sysname $Calc_Type
echo $nabc

# required executables and pseudopotentials
BIN_LIST="pw.x bands_FS.x kvecs_FS.x"
PSEUDO_LIST=" Al.vbc.UPF "

$ECHO
$ECHO "  executables directory: $BIN_DIR"
$ECHO "  pseudo directory:      $PSEUDO_DIR"
$ECHO "  temporary directory:   $TMP_DIR"
$ECHO "  checking that needed directories and files exist...\c"

# check for directories
for DIR in "$BIN_DIR" "$PSEUDO_DIR" ; do
    if test ! -d $DIR ; then 
        $ECHO
        $ECHO "ERROR: $DIR not existent or not a directory"
        $ECHO "Aborting"
        exit 1
    fi
done

# check for executables
for FILE in $BIN_LIST ; do
    if test ! -x $BIN_DIR/$FILE ; then
        $ECHO
        $ECHO "ERROR: $BIN_DIR/$FILE not existent or not executable"
        $ECHO "Aborting"
        exit 1
    fi
done

# check for pseudopotentials
for FILE in $PSEUDO_LIST ; do
    if test ! -r $PSEUDO_DIR/$FILE ; then
        $ECHO
        $ECHO "ERROR: $PSEUDO_DIR/$FILE not existent or not readable"
        $ECHO "Aborting"
        exit 1
    fi
done
$ECHO " done"

# how to run executables
PW_COMMAND="$PARA_PREFIX $BIN_DIR/pw.x $PARA_POSTFIX"
$ECHO
$ECHO "  running pw.x as: $PW_COMMAND"
$ECHO

if [ ! -d results ]; then
  mkdir results
fi  

cd results

if [ ! -d temp ]; then
  mkdir temp
fi  

#put your  scf-input file  here
#
#
# self-consistent calculation
#
cat > $Sysname.scf.in << EOF
 &control
    calculation='scf'
    restart_mode='from_scratch',
    prefix='$Sysname',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/'
    tprnfor=.true.,
    tstress=.true.
 &end
 &system    
    ibrav=  2, 
    a= 4.049, 
    nat=  1, ntyp= 1,
    ecutwfc =25.0, 
    ecutrho=300,
    occupations='smearing', 
    smearing='methfessel-paxton', 
    degauss=0.03
 &end
 &electrons
    conv_thr =  1.0d-10
    mixing_beta= 0.7
 &end
ATOMIC_SPECIES
 Al  26.98  Al.vbc.UPF
ATOMIC_POSITIONS
 Al  0.00 0.00 0.00
K_POINTS {automatic}
 16 16 16  0 0 0
EOF

   echo 'Self-consistent calculations for '$Sysname''
   $PW_COMMAND  <$Sysname.scf.in >$Sysname.scf.out
   echo 'done'

# right after SCF

a1=`grep 'b(1)' $Sysname.scf.out | cut -c 24-54`
a2=`grep 'b(2)' $Sysname.scf.out | cut -c 24-54`
a3=`grep 'b(3)' $Sysname.scf.out | cut -c 24-54`

cat > input_kvecs <<EOF
$a1
$a2
$a3
$nabc
$Sysname
EOF

E_Fermi=`grep Fermi $Sysname.scf.out | cut -c 26-36` 
echo $E_Fermi


cat > input_FS <<EOF
$n_start  $n_last
$E_Fermi
$Sysname
$Calc_Type
$nabc
$a1
$a2
$a3
EOF

$BIN_DIR/kvecs_FS.x <input_kvecs

# 
cat > $Sysname.nscf.in << EOF
 &control
    calculation='nscf'
    restart_mode='from_scratch',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    prefix='$Sysname'
 &end
 &system    
    ibrav=  2, 
    a= 4.049, 
    nat=  1, ntyp= 1,
    ecutwfc =25.0,
    ecutrho = 300, 
    occupations='smearing', 
    smearing='methfessel-paxton', 
    degauss=0.03
 &end
 &electrons
    conv_thr =  1.0d-10
 &end
ATOMIC_SPECIES
 Al  26.98  Al.vbc.UPF
ATOMIC_POSITIONS
 Al  0.00 0.00 0.00
K_POINTS 
EOF


cat  kvecs_$Sysname >> $Sysname.nscf.in

     echo 'Band-structure calculations for '$Sysname''
     $PW_COMMAND   <$Sysname.nscf.in >Bands.out
     echo 'done'
     
cp  Bands.out $Sysname.bands.out 
$PARA_PREFIX $BIN_DIR/bands_FS.x 

if [ $Calc_Type == Bands ]; then
echo $Calc_Type
mv Bands_structure.out  $Sysname.Bands.out 
 
  elif [ $Calc_Type == FS ]; then 
echo $Calc_Type
mv Bands.bxsf  $Sysname.FS.bxsf 

fi

echo 'done'


